@@ -1,14 +1,29 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+
+
 #include <boost/scoped_ptr.hpp>
 #include <boost/shared_ptr.hpp>
 #include <stdlib.h>
@@ -25,6 +40,7 @@
 
 #include "mysql_connection_options.h"
 #include "mysql_util.h"
+#include "mysql_uri.h"
 
 /*
  * _WIN32 is defined by 64bit compiler too
@@ -86,8 +102,14 @@ MySQL_Savepoint::getSavepointName()
 
 
 /* {{{ MySQL_Connection::MySQL_Connection() -I- */
-MySQL_Connection::MySQL_Connection(const sql::SQLString& hostName, const sql::SQLString& userName, const sql::SQLString& password)
-	: intern(NULL)
+MySQL_Connection::MySQL_Connection(Driver * _driver,
+								   ::sql::mysql::NativeAPI::NativeConnectionWrapper& _proxy,
+								   const sql::SQLString& hostName,
+								   const sql::SQLString& userName,
+								   const sql::SQLString& password)
+	:	driver	(_driver),
+		proxy	(&_proxy),
+		intern	(NULL)
 {
 	sql::ConnectOptionsMap connection_properties;
 	connection_properties["hostName"] = hostName;
@@ -107,8 +129,10 @@ MySQL_Connection::MySQL_Connection(const sql::SQLString& hostName, const sql::SQ
 
 
 /* {{{ MySQL_Connection::MySQL_Connection() -I- */
-MySQL_Connection::MySQL_Connection(sql::ConnectOptionsMap & properties)
-	:intern(NULL)
+MySQL_Connection::MySQL_Connection(Driver * _driver,
+								   ::sql::mysql::NativeAPI::NativeConnectionWrapper& _proxy,
+								   sql::ConnectOptionsMap & properties)
+	: driver(_driver), proxy(&_proxy), intern(NULL)
 {
 	boost::shared_ptr<MySQL_DebugLogger> tmp_logger(new MySQL_DebugLogger());
 	std::auto_ptr< MySQL_ConnectionData > tmp_intern(new MySQL_ConnectionData(tmp_logger));
@@ -182,37 +206,42 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 	CPP_ENTER_WL(intern->logger, "MySQL_Connection::init");
 
 	intern->is_valid = true;
-	bool protocol_tcp = true;
-	sql::SQLString host;
-	sql::SQLString socket_or_pipe;
-	unsigned int port = 3306;
 
-	sql::SQLString hostName;
+	MySQL_Uri uri;
+
 	sql::SQLString userName;
 	sql::SQLString password;
-	sql::SQLString schema;
 	sql::SQLString defaultCharset("utf8");
 	sql::SQLString characterSetResults("utf8");
-	// "" means default mysqlclient library name
-	sql::SQLString clientlib("");
 
 	sql::SQLString sslKey, sslCert, sslCA, sslCAPath, sslCipher;
-	bool ssl_used = false, schema_used = false;
+	bool ssl_used = false;
 	int flags = CLIENT_MULTI_RESULTS;
 
 	const int * p_i;
 	const bool * p_b;
 	const sql::SQLString * p_s;
-	
-	sql::ConnectOptionsMap::const_iterator it = properties.begin();
-	for (; it != properties.end(); ++it) {
-		if (!it->first.compare("hostName")) {
-			if ((p_s = boost::get< sql::SQLString >(&it->second))) {
-				hostName = *p_s;
-			} else {
-				throw sql::InvalidArgumentException("No string value passed for hostName");
-			}
-		} else if (!it->first.compare("userName")) {
+	bool opt_reconnect = false;
+	bool opt_reconnect_value = false;
+
+
+	/* Values set in properties individually should have priority over those
+	   we restore from Uri */
+	sql::ConnectOptionsMap::const_iterator it = properties.find("hostName");
+
+	if (it != properties.end())
+	{
+		if ((p_s = boost::get< sql::SQLString >(&it->second))) {
+            /* Parsing uri prior to processing all parameters, so indivudually
+               specified parameters precede over those in the uri */
+			parseUri(*p_s, uri);
+		} else {
+			throw sql::InvalidArgumentException("No string value passed for hostName");
+		}
+	}
+
+	for (it = properties.begin(); it != properties.end(); ++it) {
+		if (!it->first.compare("userName")) {
 			if ((p_s = boost::get< sql::SQLString >(&it->second))) {
 				userName = *p_s;
 			} else {
@@ -226,31 +255,28 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 			}
 		} else if (!it->first.compare("port")) {
 			if ((p_i = boost::get< int >(&it->second))) {
-				port = static_cast<unsigned int>(*p_i);
+				uri.setPort(static_cast<unsigned int>(*p_i));
 			} else {
 				throw sql::InvalidArgumentException("No long long value passed for port");
 			}
 		} else if (!it->first.compare("socket")) {
 			if ((p_s = boost::get< sql::SQLString >(&it->second))) {
-				socket_or_pipe = *p_s;
+				uri.setSocket(*p_s);
 			} else {
 				throw sql::InvalidArgumentException("No string value passed for socket");
 			}
-			protocol_tcp = false;
 		} else if (!it->first.compare("pipe")) {
 			if ((p_s = boost::get< sql::SQLString >(&it->second))) {
-				socket_or_pipe = *p_s;
+				uri.setPipe(*p_s);
 			} else {
 				throw sql::InvalidArgumentException("No string value passed for pipe");
 			}
-			protocol_tcp = false;
 		} else if (!it->first.compare("schema")) {
 			if ((p_s = boost::get< sql::SQLString >(&it->second))) {
-				schema = *p_s;
+				uri.setSchema(*p_s);
 			} else {
 				throw sql::InvalidArgumentException("No string value passed for schema");
 			}
-			schema_used = true;
 		} else if (!it->first.compare("characterSetResults")) {
 			if ((p_s = boost::get< sql::SQLString >(&it->second))) {
 				characterSetResults = *p_s;
@@ -292,13 +318,6 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 				throw sql::InvalidArgumentException("No string value passed for sslCipher");
 			}
 			ssl_used = true;
-		} else if (it->first.compare("clientlib") == 0) {
-			if ((p_s = boost::get< sql::SQLString >(&it->second))) {
-				clientlib = *p_s;
-			} else {
-				throw sql::InvalidArgumentException("No string value passed for driver");
-			}
-			
 		} else if (!it->first.compare("defaultStatementResultType")) {
 			if (!(p_i = boost::get< int >(&it->second))) {
 				throw sql::InvalidArgumentException("No long long value passed for defaultStatementResultType");
@@ -406,52 +425,10 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 		}
 	}
 
-	try {
-		proxy.reset(NativeAPI::createNativeConnectionWrapper(clientlib));
-	} catch (std::runtime_error & e) {
-		throw sql::InvalidArgumentException(e.what());
-	}
 
-#ifndef _WIN32
-	if (!hostName.compare(0, sizeof("unix://") - 1, "unix://")) {
-		protocol_tcp = false;
-		socket_or_pipe = hostName.substr(sizeof("unix://") - 1, sql::SQLString::npos);
-		host = "localhost";
-		int tmp_protocol = MYSQL_PROTOCOL_SOCKET;
-		proxy->options(MYSQL_OPT_PROTOCOL, (const char *) &tmp_protocol);
-	} else
-#else
-	if (!hostName.compare(0, sizeof("pipe://") - 1, "pipe://")) {
-		protocol_tcp = false;
-		socket_or_pipe = hostName.substr(sizeof("pipe://") - 1, sql::SQLString::npos);
-		host = ".";
-		int tmp_protocol = MYSQL_PROTOCOL_PIPE;
-		proxy->options(MYSQL_OPT_PROTOCOL, (const char *) &tmp_protocol);
-	} else
-#endif
-	if (!hostName.compare(0, sizeof("tcp://") - 1, "tcp://") ) {
-		size_t port_pos, schema_pos;
-		host = hostName.substr(sizeof("tcp://") - 1, sql::SQLString::npos);
-		schema_pos = host.find('/');
-		if (schema_pos != sql::SQLString::npos) {
-			++schema_pos; // skip the slash
-							/* TODO: tcp://127.0.0.1/
-							-> host set, schema empty, schema property ignored */
-			schema = host.substr(schema_pos, host.length() - schema_pos);
-			schema_used = true;
-			host = host.substr(0, schema_pos - 1);
-		}
-		port_pos = host.find_last_of(':', sql::SQLString::npos);
-		if (port_pos != sql::SQLString::npos) {
-		port = atoi(host.substr(port_pos + 1, sql::SQLString::npos).c_str());
-			host = host.substr(0, port_pos);
-		}
-	} else {
-		host = hostName.c_str();
-	}
 	/* libmysql shouldn't think it is too smart */
-	if (protocol_tcp && !host.compare(0, sizeof("localhost") - 1, "localhost")) {
-		host = "127.0.0.1";
+	if (tcpProtocol(uri) && !uri.Host().compare(util::LOCALHOST)) {
+		uri.setHost("127.0.0.1");
 	}
 
 	it = properties.begin();
@@ -478,7 +455,8 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 			if (!(p_b = boost::get<bool>(&it->second))) {
 				throw sql::InvalidArgumentException("No bool value passed for OPT_RECONNECT");
 			}
-			proxy->options(MYSQL_OPT_RECONNECT, (const char *) &p_b);
+			opt_reconnect = true;
+			opt_reconnect_value = *p_b;
 		} else if (!it->first.compare("OPT_CHARSET_NAME")) {
 			if (!(p_s = boost::get< sql::SQLString >(&it->second))) {
 				throw sql::InvalidArgumentException("No SQLString value passed for OPT_CHARSET_NAME");
@@ -488,36 +466,47 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 			if (!(p_b = boost::get< bool >(&it->second))) {
 				throw sql::InvalidArgumentException("No bool value passed for OPT_REPORT_DATA_TRUNCATION");
 			}
-			proxy->options(MYSQL_REPORT_DATA_TRUNCATION, (const char *) &p_b);
-#ifdef _WIN32
+			proxy->options(MYSQL_REPORT_DATA_TRUNCATION, (const char *) p_b);
 		} else if (!it->first.compare("OPT_NAMED_PIPE")) {
-			proxy->options(MYSQL_OPT_NAMED_PIPE, NULL);
-#endif
+			/* Not sure it is really needed */
+			uri.setProtocol(NativeAPI::PROTOCOL_PIPE);
 		}
 	}
 
+// Throwing in case of wrong protocol
+#ifdef _WIN32
+	if (uri.Protocol() == NativeAPI::PROTOCOL_SOCKET) {
+		throw sql::InvalidArgumentException("Invalid for this platform protocol requested(MYSQL_PROTOCOL_SOCKET)");
+	}
+#else
+	if (uri.Protocol() == NativeAPI::PROTOCOL_PIPE) {
+		throw sql::InvalidArgumentException("Invalid for this platform protocol requested(MYSQL_PROTOCOL_PIPE)");
+	}
+#endif
+	proxy->use_protocol(uri.Protocol());
+
 	{
 		my_bool tmp_bool = 1;
 		proxy->options(MYSQL_SECURE_AUTH, (const char *) &tmp_bool);
 	}
 
 	proxy->options(MYSQL_SET_CHARSET_NAME, defaultCharset.c_str());
-	
+
 	if (ssl_used) {
 		/* According to the docs, always returns 0 */
 		proxy->ssl_set(sslKey.c_str(), sslCert.c_str(), sslCA.c_str(), sslCAPath.c_str(), sslCipher.c_str());
 	}
-	CPP_INFO_FMT("hostName=%s", hostName.c_str());
+	CPP_INFO_FMT("hostName=%s", uri.Host().c_str());
 	CPP_INFO_FMT("user=%s", userName.c_str());
-	CPP_INFO_FMT("port=%d", port);
-	CPP_INFO_FMT("schema=%s", schema.c_str());
-	CPP_INFO_FMT("socket/pipe=%s", socket_or_pipe.c_str());
-	if (!proxy->connect(host,
+	CPP_INFO_FMT("port=%d", uri.Port());
+	CPP_INFO_FMT("schema=%s", uri.Schema().c_str());
+	CPP_INFO_FMT("socket/pipe=%s", uri.SocketOrPipe().c_str());
+	if (!proxy->connect(uri.Host(),
 						userName,
 						password,
-						schema_used && schema.length()? schema: (schema= "") /* schema */,
-						port,
-						protocol_tcp == false? socket_or_pipe : "" /*socket or named pipe */,
+						uri.Schema() /* schema */,
+						uri.Port(),
+						uri.SocketOrPipe() /*socket or named pipe */,
 						flags))
 	{
 		CPP_ERR_FMT("Couldn't connect : %d", proxy->errNo());
@@ -528,6 +517,10 @@ void MySQL_Connection::init(ConnectOptionsMap & properties)
 		proxy.reset();
 		throw e;
 	}
+	if (opt_reconnect) {
+		proxy->options(MYSQL_OPT_RECONNECT, (const char *) &opt_reconnect_value);
+	}
+
 	setAutoCommit(true);
 	setTransactionIsolation(sql::TRANSACTION_REPEATABLE_READ);
 	// Different Values means we have to set different result set encoding
@@ -544,6 +537,7 @@ void
 MySQL_Connection::clearWarnings()
 {
 	CPP_ENTER_WL(intern->logger, "MySQL_Connection::clearWarnings");
+
 	intern->warnings.reset();
 }
 /* }}} */
@@ -568,6 +562,7 @@ MySQL_Connection::close()
 	CPP_ENTER_WL(intern->logger, "MySQL_Connection::close");
 	checkClosed();
 	proxy.reset();
+	clearWarnings();
 	intern->is_valid = false;
 }
 /* }}} */
@@ -594,6 +589,16 @@ sql::Statement * MySQL_Connection::createStatement()
 /* }}} */
 
 
+/* {{{ MySQL_Connection::escapeString() -I- */
+sql::SQLString MySQL_Connection::escapeString(const sql::SQLString & s)
+{
+	CPP_ENTER_WL(intern->logger, "MySQL_Connection::escapeString");
+	checkClosed();
+	return proxy->escapeString(s);
+}
+/* }}} */
+
+
 /* {{{ MySQL_Connection::getAutoCommit() -I- */
 bool
 MySQL_Connection::getAutoCommit()
@@ -616,6 +621,14 @@ MySQL_Connection::getCatalog()
 /* }}} */
 
 
+/* {{{ MySQL_Connection::getDriver() -I- */
+Driver * MySQL_Connection::getDriver()
+{
+	return driver;
+}
+/* }}} */
+
+
 /**
   Added for consistency. Not present in jdbc interface. Is still subject for discussion.
 */
@@ -691,6 +704,8 @@ MySQL_Connection::getWarnings()
 	CPP_ENTER_WL(intern->logger, "MySQL_Connection::getWarnings");
 	checkClosed();
 
+	clearWarnings();
+
 	intern->warnings.reset(loadMysqlWarnings(this));
 
 	return intern->warnings.get();
@@ -703,7 +718,13 @@ bool
 MySQL_Connection::isClosed()
 {
 	CPP_ENTER_WL(intern->logger, "MySQL_Connection::isClosed");
-	return !intern->is_valid;
+	if (intern->is_valid) {
+		if (!proxy->ping()) {
+			return false;
+		}
+		close();
+	}
+	return true;
 }
 /* }}} */
 
@@ -886,8 +907,8 @@ MySQL_Connection::setSchema(const sql::SQLString& catalog)
 {
 	CPP_ENTER_WL(intern->logger, "MySQL_Connection::setCatalog");
 	checkClosed();
-	sql::SQLString sql("USE ");
-	sql.append(catalog);
+	sql::SQLString sql("USE `");
+	sql.append(catalog).append("`");
 
 	boost::scoped_ptr< sql::Statement > stmt(createStatement());
 	stmt->execute(sql);
@@ -1039,6 +1060,18 @@ MySQL_Connection::setTransactionIsolation(enum_transaction_isolation level)
 /* }}} */
 
 
+/* {{{ MySQL_Connection::getLastStatementInfo() -I- */
+sql::SQLString
+MySQL_Connection::getLastStatementInfo()
+{
+	CPP_ENTER_WL(intern->logger, "MySQL_Connection::getLastStatementInfo");
+	checkClosed();
+
+	return proxy->info();
+}
+/* }}} */
+
+
 /* {{{ MySQL_Connection::getSessionVariable() -I- */
 sql::SQLString
 MySQL_Connection::getSessionVariable(const sql::SQLString & varname)
