@@ -1,14 +1,29 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+
+
 #ifndef _MYSQL_CONNECTION_DATA_H_
 #define _MYSQL_CONNECTION_DATA_H_
 
@@ -18,7 +33,7 @@
 #include <cppconn/resultset.h>
 #include "mysql_util.h"
 #include "mysql_metadata.h"
-struct st_mysql;
+#include "mysql_warning.h"
 
 namespace sql
 {
@@ -39,23 +54,15 @@ public:
 		  defaultPreparedStatementResultType(sql::ResultSet::TYPE_SCROLL_INSENSITIVE),
 		  logger(l), meta(NULL) {}
 
-	~MySQL_ConnectionData() {}
+	~MySQL_ConnectionData()
+	{
+	}
 
 	bool closed;
 	bool autocommit;
 	enum_transaction_isolation txIsolationLevel;
 
-	/* disable compile warnings on Windows */
-#ifdef _WIN32
-#pragma warning(push)
-#pragma warning (disable : 4251)
-#endif
-
-	boost::scoped_ptr<const sql::SQLWarning> warnings;
-
-#ifdef _WIN32
-#pragma warning (pop)
-#endif
+	boost::scoped_ptr<const MySQL_Warning> warnings;
 
 	bool is_valid;
 
