@@ -1,14 +1,29 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+
+
 #include <memory>
 #include <algorithm>
 
@@ -38,7 +53,8 @@ namespace mysql
 /* {{{ MySQL_Statement::MySQL_Statement() -I- */
 MySQL_Statement::MySQL_Statement(MySQL_Connection * conn, boost::shared_ptr< NativeAPI::NativeConnectionWrapper > & _proxy,
 									sql::ResultSet::enum_type rset_type, boost::shared_ptr< MySQL_DebugLogger > & l)
-	: warnings(NULL), connection(conn), proxy(_proxy), isClosed(false), last_update_count(UL64(~0)), logger(l), resultset_type(rset_type)
+	: warnings(NULL), connection(conn), proxy(_proxy), isClosed(false), warningsHaveBeenLoaded(true),
+  last_update_count(UL64(~0)), logger(l), resultset_type(rset_type), warningsCount(0)
 {
 	CPP_ENTER("MySQL_Statement::MySQL_Statement");
 	CPP_INFO_FMT("this=%p", this);
@@ -52,10 +68,7 @@ MySQL_Statement::~MySQL_Statement()
 	CPP_ENTER("MySQL_Statement::~MySQL_Statement");
 	CPP_INFO_FMT("this=%p", this);
 
-	for (sql::SQLWarning * tmp = warnings, * next_tmp = warnings; tmp; tmp = next_tmp) {
-		next_tmp = const_cast<sql::SQLWarning *>(tmp->getNextWarning());
-		delete tmp;
-	}
+	warnings.reset();
 }
 /* }}} */
 
@@ -72,6 +85,10 @@ MySQL_Statement::do_query(const char *q, size_t length)
 		CPP_ERR_FMT("Error during proxy->query : %d:(%s) %s", proxy->errNo(), proxy->sqlstate().c_str(), proxy->error().c_str());
 		sql::mysql::util::throwSQLException(*proxy.get());
 	}
+
+	warningsCount= proxy->warning_count();
+
+	warningsHaveBeenLoaded= false;
 }
 /* }}} */
 
@@ -85,19 +102,19 @@ MySQL_Statement::get_resultset()
 	checkClosed();
 
 	NativeAPI::NativeResultsetWrapper * result;
-	//TODO: again - probably no need to catch-n-throw here. O maybe no need to throw further
-	try
-	{
+	//TODO: again - probably no need to catch-n-throw here. Or maybe no need to throw further
+	try {
 		result= (resultset_type == sql::ResultSet::TYPE_FORWARD_ONLY)? proxy->use_result(): proxy->store_result();
-	}
-	catch (::sql::SQLException & e)
-	{
+		if (!result) {
+			sql::mysql::util::throwSQLException(*proxy.get());
+		}
+	} catch (::sql::SQLException & e) {
 		CPP_ERR_FMT("Error during %s_result : %d:(%s) %s", resultset_type == sql::ResultSet::TYPE_FORWARD_ONLY? "use":"store",
 			proxy->errNo(), proxy->sqlstate().c_str(), proxy->error().c_str());
 		throw e;
 	}
 
-	return boost::shared_ptr< NativeAPI::NativeResultsetWrapper >( result );
+	return boost::shared_ptr< NativeAPI::NativeResultsetWrapper >(result);
 }
 /* }}} */
 
@@ -205,23 +222,38 @@ MySQL_Statement::getResultSet()
 
 	last_update_count = UL64(~0);
 
-
 	boost::shared_ptr< NativeAPI::NativeResultsetWrapper > result;
 
 	sql::ResultSet::enum_type tmp_type;
 
 	try {
+		NativeAPI::NativeResultsetWrapper * tmp_ptr;
 		switch (resultset_type) {
 			case sql::ResultSet::TYPE_FORWARD_ONLY:
-				result.reset(proxy->use_result());
+				if (!(tmp_ptr = proxy->use_result())) {
+					sql::mysql::util::throwSQLException(*proxy.get());
+				}
+				result.reset(tmp_ptr);
 				tmp_type = sql::ResultSet::TYPE_FORWARD_ONLY;
 				break;
 			default:
-				result.reset(proxy->store_result());
+				if (!(tmp_ptr = proxy->store_result())) {
+					sql::mysql::util::throwSQLException(*proxy.get());
+				}
+				result.reset(tmp_ptr);
 				tmp_type = sql::ResultSet::TYPE_SCROLL_INSENSITIVE;
 		}
-	} catch (::sql::SQLException & /*e*/ ) {
-		return NULL;
+	} catch (::sql::SQLException & e ) {
+		if (proxy->errNo() != 0)
+		{
+			CPP_ERR_FMT("Error during %s_result : %d:(%s) %s", resultset_type == sql::ResultSet::TYPE_FORWARD_ONLY? "use":"store",
+				proxy->errNo(), proxy->sqlstate().c_str(), proxy->error().c_str());
+			throw e;
+		}
+		else
+		{
+			return NULL;
+		}
 	}
 
 	if (!result) {
@@ -268,11 +300,7 @@ MySQL_Statement::clearWarnings()
 	CPP_ENTER("MySQL_Statement::clearWarnings");
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
-	for (sql::SQLWarning * tmp = warnings, * next_tmp = warnings; tmp; tmp = next_tmp) {
-		next_tmp = const_cast<sql::SQLWarning *>(tmp->getNextWarning());
-		delete tmp;
-	}
-	warnings = NULL;
+	warnings.reset();
 }
 /* }}} */
 
@@ -284,10 +312,7 @@ MySQL_Statement::close()
 	CPP_ENTER("MySQL_Statement::close");
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
-	for (sql::SQLWarning * tmp = warnings, * next_tmp = warnings; tmp; tmp = next_tmp) {
-		next_tmp = const_cast<sql::SQLWarning *>(tmp->getNextWarning());
-		delete tmp;
-	}
+	clearWarnings();
 	isClosed = true;
 }
 /* }}} */
@@ -389,7 +414,13 @@ MySQL_Statement::getWarnings()
 	CPP_INFO_FMT("this=%p", this);
 	checkClosed();
 
-	return warnings;
+	if (!warningsHaveBeenLoaded)
+	{
+		warnings.reset(loadMysqlWarnings(connection, warningsCount));
+		warningsHaveBeenLoaded= true;
+	}
+
+	return warnings.get();
 }
 /* }}} */
 
