@@ -1,20 +1,38 @@
 /*
-   Copyright 2007 - 2008 MySQL AB, 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
+
+
 #ifndef _MYSQL_STATEMENT_H_
 #define _MYSQL_STATEMENT_H_
 
 #include <boost/shared_ptr.hpp>
 #include <cppconn/statement.h>
 #include <cppconn/resultset.h>
+
+#include "mysql_warning.h"
+
 #include "mysql_util.h"
 
 namespace sql
@@ -34,12 +52,13 @@ class NativeConnectionWrapper;
 class MySQL_Statement : public sql::Statement
 {
 protected:
-	sql::SQLWarning * warnings;
+	boost::scoped_ptr<MySQL_Warning> warnings;
 	MySQL_Connection * connection;
 	boost::shared_ptr< NativeAPI::NativeConnectionWrapper > proxy;
 
 	void do_query(const char *q, size_t length);
 	bool isClosed;
+	bool warningsHaveBeenLoaded;
 
 	uint64_t last_update_count;
 
@@ -47,6 +66,8 @@ protected:
 
 	sql::ResultSet::enum_type resultset_type;
 
+	unsigned int warningsCount;
+
 	virtual boost::shared_ptr< NativeAPI::NativeResultsetWrapper > get_resultset();
 	virtual void checkClosed();
 
