@@ -1,13 +1,28 @@
 /*
-   Copyright 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
+
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
- */
 
 #include <cppconn/warning.h>
 
@@ -454,7 +469,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["hostName"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception I");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -468,7 +483,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["userName"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception II");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -482,7 +497,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["password"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception III");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -496,7 +511,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["port"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception IV");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -509,7 +524,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["socket"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception V");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -522,7 +537,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["pipe"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception VI");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -535,7 +550,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["schema"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception VII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -548,7 +563,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["characterSetResults"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception VIII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -561,7 +576,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["sslKey"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception IX");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -574,7 +589,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["sslCert"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception X");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -587,7 +602,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["sslCA"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XI");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -600,7 +615,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["sslCAPath"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -613,7 +628,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["sslCipher"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XIII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -621,25 +636,28 @@ void connection::connectUsingMapWrongTypes()
     }
     connection_properties.erase("sslCipher");
 
+    /*
+    TODO -- will be moved into driver class.
     try
     {
       connection_properties["clientlib"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XIV");
     }
     catch (sql::InvalidArgumentException)
     {
-      /* expected */
+      expected
     }
     connection_properties.erase("clientlib");
+    */
 
     try
     {
       connection_properties["defaultStatementResultType"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XV");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -652,7 +670,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["metadataUseInfoSchema"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XVI");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -665,7 +683,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["CLIENT_COMPRESS"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XVII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -678,7 +696,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["CLIENT_FOUND_ROWS"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XVIII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -691,7 +709,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["CLIENT_IGNORE_SIGPIPE"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XIX");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -704,7 +722,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["CLIENT_IGNORE_SPACE"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XX");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -717,7 +735,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["CLIENT_INTERACTIVE"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXI");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -730,20 +748,20 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["CLIENT_LOCAL_FILES"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXII");
     }
     catch (sql::InvalidArgumentException)
     {
       /* expected */
     }
     connection_properties.erase("CLIENT_LOCAL_FILES");
-    
+
     try
     {
       connection_properties["CLIENT_MULTI_STATEMENTS"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXIII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -756,7 +774,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["CLIENT_NO_SCHEMA"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXIV");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -769,7 +787,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["OPT_CONNECT_TIMEOUT"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXV");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -782,7 +800,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["OPT_READ_TIMEOUT"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXVI");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -795,7 +813,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["OPT_WRITE_TIMEOUT"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXVII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -808,7 +826,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["OPT_RECONNECT"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXVIII");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -821,7 +839,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["OPT_CHARSET_NAME"]=(boolval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXIX");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -834,7 +852,7 @@ void connection::connectUsingMapWrongTypes()
       connection_properties["OPT_REPORT_DATA_TRUNCATION"]=(strval);
       created_objects.clear();
       con.reset(driver->connect(connection_properties));
-      FAIL("No exception");
+      FAIL("No exception XXX");
     }
     catch (sql::InvalidArgumentException)
     {
@@ -1723,7 +1741,7 @@ void connection::connectUsingMap()
         created_objects.clear();
         con.reset(driver->connect(connection_properties));
       }
-      catch (sql::SQLException &e)
+      catch (sql::SQLException &/*e*/)
       {
         /* may not be compiled in - ignore */
       }
@@ -1740,6 +1758,196 @@ void connection::connectUsingMap()
 
 }
 
+void connection::connectOptReconnect()
+{
+  logMsg("connection::connectOptReconnect - OPT_RECONNECT");
+  std::stringstream msg;
+
+  try
+  {
+    sql::ConnectOptionsMap connection_properties;
+
+    connection_properties["hostName"]=url;
+    connection_properties["userName"]=user;
+    connection_properties["password"]=passwd;
+
+    bool bval= !TestsRunner::getStartOptions()->getBool("dont-use-is");
+    connection_properties["metadataUseInfoSchema"]=(bval);
+
+    logMsg("... OPT_RECONNECT disabled");
+
+    connection_properties.erase("OPT_RECONNECT");
+    connection_properties["OPT_RECONNECT"]=false;
+
+    created_objects.clear();
+    con.reset(driver->connect(connection_properties));
+    con->setSchema(db);
+
+    con->close();
+    ASSERT(con->isClosed());
+    try
+    {
+      stmt.reset(con->createStatement());
+      stmt->execute("DROP TABLE IF EXISTS test");
+      FAIL("Can create statement although connection has been closed");
+    }
+    catch (sql::SQLException &/*e*/)
+    {
+      /* expected */
+    }
+
+    connection_properties.erase("OPT_RECONNECT");
+    connection_properties["OPT_RECONNECT"]=false;
+
+    created_objects.clear();
+    con.reset(driver->connect(connection_properties));
+    con->setSchema(db);
+
+    ASSERT_EQUALS(false, con->isClosed());
+    try
+    {
+      stmt.reset(con->createStatement());
+      stmt->execute("DROP TABLE IF EXISTS test");
+    }
+    catch (sql::SQLException &e)
+    {
+      logErr(e.what());
+      logErr("SQLState: " + std::string(e.getSQLState()));
+      fail(e.what(), __FILE__, __LINE__);
+    }
+
+    logMsg("... OPT_RECONNECT enabled");
+
+    connection_properties.erase("OPT_RECONNECT");
+    connection_properties["OPT_RECONNECT"]=true;
+
+    created_objects.clear();
+    con.reset(driver->connect(connection_properties));
+    con->setSchema(db);
+
+    con->close();
+    ASSERT(con->isClosed());
+    try
+    {
+      stmt.reset(con->createStatement());
+      stmt->execute("DROP TABLE IF EXISTS test");
+      FAIL("Can create statement although connection has been closed");
+    }
+    catch (sql::SQLException &/*e*/)
+    {
+      /* expected */
+    }
+
+    connection_properties.erase("OPT_RECONNECT");
+    connection_properties["OPT_RECONNECT"]=true;
+
+    created_objects.clear();
+    con.reset(driver->connect(connection_properties));
+    con->setSchema(db);
+
+    ASSERT_EQUALS(false, con->isClosed());
+    try
+    {
+      stmt.reset(con->createStatement());
+      stmt->execute("DROP TABLE IF EXISTS test");
+    }
+    catch (sql::SQLException &e)
+    {
+      logErr(e.what());
+      logErr("SQLState: " + std::string(e.getSQLState()));
+      fail(e.what(), __FILE__, __LINE__);
+    }
+
+    logMsg("... OPT_RECONNECT disabled and KILL");
+
+
+    connection_properties.erase("OPT_RECONNECT");
+    connection_properties["OPT_RECONNECT"]=false;
+
+    created_objects.clear();
+    con.reset(driver->connect(connection_properties));
+    con->setSchema(db);
+    stmt.reset(con->createStatement());
+    res.reset(stmt->executeQuery("SELECT CONNECTION_ID() as _pid"));
+    ASSERT(res->next());
+    msg.str("");
+    msg << "KILL " << res->getInt("_pid");
+
+    try
+    {
+      Connection my_con(getConnection());
+      my_con->setSchema(db);
+      Statement my_stmt(my_con->createStatement());
+      my_stmt->execute(msg.str());
+      logMsg("... we seem to be lucky, we have killed a connection");
+      logMsg(msg.str());
+      try
+      {
+        msg.str("");
+        msg << "USE " << db;
+        stmt->execute(msg.str());
+        stmt->execute("DROP TABLE IF EXISTS test");
+        FAIL("Statement object is still usable");
+      }
+      catch (sql::SQLException &e)
+      {
+        // Any error message is fine, connection should have been killed
+        logMsg(e.what());
+      }
+    }
+    catch (sql::SQLException &/*e*/)
+    {
+      // KILL has failed - that is OK, we may not have permissions
+    }
+
+    connection_properties.erase("OPT_RECONNECT");
+    connection_properties["OPT_RECONNECT"]=false;
+
+    created_objects.clear();
+    con.reset(driver->connect(connection_properties));
+    con->setSchema(db);
+    stmt.reset(con->createStatement());
+    res.reset(stmt->executeQuery("SELECT CONNECTION_ID() as _pid"));
+    ASSERT(res->next());
+    msg.str("");
+    msg << "KILL " << res->getInt("_pid");
+
+    try
+    {
+      Connection my_con(getConnection());
+      my_con->setSchema(db);
+      Statement my_stmt(my_con->createStatement());
+      my_stmt->execute(msg.str());
+      logMsg("... we seem to be lucky, we have killed a connection");
+      logMsg(msg.str());
+      try
+      {
+        stmt.reset(con->createStatement());
+        logMsg("... we got a new statement object");
+        stmt->execute("DROP TABLE IF EXISTS test");
+        FAIL("Statement object is still usable");
+      }
+      catch (sql::SQLException &e)
+      {
+        /* Any error message is fine, connection should have been killed */
+        logMsg(e.what());
+      }
+    }
+    catch (sql::SQLException &/*e*/)
+    {
+      /* KILL has failed - that is OK, we may not have permissions */
+    }
+
+  }
+  catch (sql::SQLException &e)
+  {
+    logErr(e.what());
+    logErr("SQLState: " + std::string(e.getSQLState()));
+    fail(e.what(), __FILE__, __LINE__);
+  }
+
+}
+
 void connection::setTransactionIsolation()
 {
   logMsg("connection::setTransactionIsolation() - MySQL_Connection::setTransactionIsolation()");
