@@ -1,13 +1,28 @@
 /*
-   Copyright 2008 - 2009 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
+
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
- */
 
 #include "../unit_fixture.h"
 
@@ -39,6 +54,7 @@ public:
     TEST_CASE(checkClosed);
     TEST_CASE(connectUsingMap);
     TEST_CASE(connectUsingMapWrongTypes);
+    TEST_CASE(connectOptReconnect);
     TEST_CASE(invalidCredentials);
     TEST_CASE(setTransactionIsolation);
     TEST_CASE(rollback);
@@ -114,6 +130,13 @@ public:
    */
   void connectUsingMap();
 
+  /*
+   * OPT_RECONNECT
+   *
+   * Check if it does what it is supposed to do
+   */
+  void connectOptReconnect();
+
   /*
    * Test of MySQL_Connection::setSessionVariable()
    *
@@ -126,7 +149,7 @@ public:
   /**
    * Establish a connection with invalid credentials
    *
-   * 
+   *
    */
   void invalidCredentials();
 
