@@ -1,13 +1,28 @@
 /*
-   Copyright 2008 - 2010 Sun Microsystems, Inc.  All rights reserved.
+Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+
+The MySQL Connector/C++ is licensed under the terms of the GPLv2
+<http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
+MySQL Connectors. There are special exceptions to the terms and
+conditions of the GPLv2 as it is applied to this software, see the
+FLOSS License Exception
+<http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published
+by the Free Software Foundation; version 2 of the License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License along
+with this program; if not, write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
+
 
-   The MySQL Connector/C++ is licensed under the terms of the GPL
-   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-   MySQL Connectors. There are special exceptions to the terms and
-   conditions of the GPL as it is applied to this software, see the
-   FLOSS License Exception
-   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
- */
 
 #include <cppconn/warning.h>
 
@@ -267,7 +282,7 @@ void connectionmetadata::getColumnPrivileges()
     {
       rows++;
 
-      if (con->getCatalog() != res->getString(1))
+      if (con->getCatalog() != "" && res->getString(1) != "" && con->getCatalog() != res->getString(1))
       {
         got_warning=true;
         msg.str("");
@@ -362,7 +377,7 @@ void connectionmetadata::getColumns()
       res.reset(dbmeta->getColumns(con->getCatalog(), con->getSchema(), "test", "id"));
       checkResultSetScrolling(res);
       ASSERT_EQUALS(true, res->next());
-      if (con->getCatalog() != res->getString("TABLE_CAT"))
+      if (con->getCatalog() != "" && res->getString(1) != "" && con->getCatalog() != res->getString("TABLE_CAT"))
       {
         got_todo_warning=true;
         msg.str();
@@ -539,15 +554,7 @@ void connectionmetadata::getDatabaseVersions()
     DatabaseMetaData * dbmeta=con->getMetaData();
     ASSERT_GT(2, dbmeta->getDatabaseMajorVersion());
     ASSERT_LT(7, dbmeta->getDatabaseMajorVersion());
-
-    if (dbmeta->getDatabaseMinorVersion() < 0)
-      FAIL("getDatabaseMinorVersion returns negative value.");
-
     ASSERT_LT(100, dbmeta->getDatabaseMinorVersion());
-
-    if (dbmeta->getDatabasePatchVersion() < 0)
-      FAIL("getDatabasePatchVersuin returns negative value.");
-
     ASSERT_LT(100, dbmeta->getDatabasePatchVersion());
 
     ASSERT_EQUALS("MySQL", dbmeta->getDatabaseProductName());
@@ -865,7 +872,7 @@ void connectionmetadata::checkForeignKey(Connection &mycon, ResultSet &myres)
   bool got_warning=false;
   std::stringstream msg;
 
-  if (mycon->getCatalog() != myres->getString(1))
+  if (mycon->getCatalog() != "" && mycon->getCatalog() != myres->getString(1))
   {
     got_warning=true;
     msg.str("");
@@ -884,7 +891,7 @@ void connectionmetadata::checkForeignKey(Connection &mycon, ResultSet &myres)
   ASSERT_EQUALS("pid", myres->getString(4));
   ASSERT_EQUALS(myres->getString(4), myres->getString("PKCOLUMN_NAME"));
 
-  if (mycon->getCatalog() != myres->getString(5))
+  if (mycon->getCatalog() != "" && mycon->getCatalog() != myres->getString(5))
   {
     got_warning=true;
     msg.str("");
@@ -958,7 +965,7 @@ void connectionmetadata::getIndexInfo()
     stmt->execute("INSERT INTO test(col1, col2, col3) VALUES (1, 1, 1)");
     res.reset(dbmeta->getIndexInfo(con->getCatalog(), con->getSchema(), "test", false, false));
     ASSERT(res->next());
-    if (con->getCatalog() != res->getString(1))
+    if (con->getCatalog() != "" && res->getString(1) != "" && con->getCatalog() != res->getString(1))
     {
       got_todo_warning=true;
       msg.str("");
@@ -1357,7 +1364,7 @@ void connectionmetadata::getPrimaryKeys()
     while (res->next())
     {
       row_num++;
-      if (con->getCatalog() != res->getString("TABLE_CAT"))
+      if (con->getCatalog() != "" && res->getString("TABLE_CAT") != "" && con->getCatalog() != res->getString("TABLE_CAT"))
       {
         got_warning=true;
         msg.str("");
@@ -1446,7 +1453,7 @@ void connectionmetadata::getProcedures()
     logMsg("...is it you, getProcedures()?");
     ASSERT(res->next());
 
-    if (con->getCatalog() != res->getString("PROCEDURE_CAT"))
+    if (con->getCatalog() != "" && res->getString("PROCEDURE_CAT") != "" && con->getCatalog() != res->getString("PROCEDURE_CAT"))
     {
       got_warning=true;
       msg.str("");
@@ -1758,7 +1765,9 @@ void connectionmetadata::getSQLKeywords()
     "UNSIGNED, UPDATE, USAGE, USE, USING, UTC_DATE,"\
     "UTC_TIME, UTC_TIMESTAMP, VALUES, VARBINARY, VARCHAR,"\
     "VARCHARACTER, VARYING, WHEN, WHERE, WHILE, WITH,"\
-    "WRITE, X509, XOR, YEAR_MONTH, ZEROFILL");
+    "WRITE, X509, XOR, YEAR_MONTH, ZEROFILL" \
+    "GENERAL, IGNORE_SERVER_IDS, MASTER_HEARTBEAT_PERIOD," \
+    "MAXVALUE, RESIGNAL, SIGNAL, SLOW");
 
   try
   {
